//+------------------------------------------------------------------+
//|                                                    Predictor.mq5 |
//|                                               Copyright VDV Soft |
//|                                                 vdv_2001@mail.ru |
//+------------------------------------------------------------------+
#property copyright "VDV Soft"
#property link      "vdv_2001@mail.ru"
#property version   "1.00"

#include <Arrays\ArrayString.mqh>

#property indicator_chart_window
#property indicator_buffers 5
#property indicator_plots   1
//---- plot buffer 1
#property indicator_label1  "Open;High;Low;Close"
#property indicator_type1   DRAW_COLOR_CANDLES
#property indicator_color1  Blue,Green,Red
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1

#define COUNT_PERIOD 7

//---- input variables
input bool     InpMarketWatch=true; //  " "  
input double   InpCrl=0.95;          // 
input int      InpNumBarsHist=10;          // 
//--- indicator buffers
double ExtOpenBuffer[];
double ExtHighBuffer[];
double ExtLowBuffer[];
double ExtCloseBuffer[];
double ExtColorsBuffer[];
//---- public variables
ENUM_TIMEFRAMES   period2[COUNT_PERIOD]=
  {
   PERIOD_M1,PERIOD_M5,PERIOD_M15,PERIOD_M30,PERIOD_H1,PERIOD_H4,PERIOD_D1 
  };
double            bh[COUNT_PERIOD],bl[COUNT_PERIOD];

CArrayString      SymbolList;
string            corr_instr_arr;
string            comm;
//--- indicators handles

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
   int count=SymbolsTotal(InpMarketWatch);
   SymbolList.Clear();
   for(int i=0;i<count;i++)
      SymbolList.Add(SymbolName(i,InpMarketWatch));
   if(SymbolList.Total()<=0)
      Print("Unable to get symbol list!");
//--- indicator buffers mapping
   SetIndexBuffer(0,ExtOpenBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,ExtHighBuffer,INDICATOR_DATA);
   SetIndexBuffer(2,ExtLowBuffer,INDICATOR_DATA);
   SetIndexBuffer(3,ExtCloseBuffer,INDICATOR_DATA);
   SetIndexBuffer(4,ExtColorsBuffer,INDICATOR_COLOR_INDEX);
//--- set accuracy
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits);
//--- sets first bar from what index will be drawn
   PlotIndexSetInteger(0,PLOT_SHIFT,COUNT_PERIOD+1);
   
   PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,EMPTY_VALUE);
   PlotIndexSetDouble(1,PLOT_EMPTY_VALUE,EMPTY_VALUE);
   PlotIndexSetDouble(2,PLOT_EMPTY_VALUE,EMPTY_VALUE);
   PlotIndexSetDouble(3,PLOT_EMPTY_VALUE,EMPTY_VALUE);
   PlotIndexSetDouble(4,PLOT_EMPTY_VALUE,EMPTY_VALUE);
//---
//for(int i=0;i<SymbolList.Total();i++)
//   Print(SymbolList.At(i));
   return(0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double iOpen(string symbol,ENUM_TIMEFRAMES period,int index)
  {
   double result;
   double iOpen[];
   if(CopyOpen(symbol,period,index,1,iOpen)!=1)
     {
      return(0);
     }
   result=iOpen[0];
   ArrayFree(iOpen);
   return(result);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   int shift_bars=rates_total-COUNT_PERIOD;
   comm="-  "+string(SymbolList.Total())+"\n";
   int prev=prev_calculated;
   if(NewBar(PERIOD_M1)||prev)
     {
      prev=0;
      for(int i=0; i<10;i++)
      {
         ExtOpenBuffer[shift_bars-i]=EMPTY_VALUE;
         ExtHighBuffer[shift_bars-i]=EMPTY_VALUE;
         ExtLowBuffer[shift_bars-i]=EMPTY_VALUE;
         ExtCloseBuffer[shift_bars-i]=EMPTY_VALUE;
      }
      //ArrayInitialize(ExtOpenBuffer,EMPTY_VALUE);
      //ArrayInitialize(ExtHighBuffer,EMPTY_VALUE);
      //ArrayInitialize(ExtLowBuffer,EMPTY_VALUE);
      //ArrayInitialize(ExtCloseBuffer,EMPTY_VALUE);
      PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,shift_bars);
     }
   corr_instr_arr="";
   int colorbar;
   string buf;
   int signal_buy=0,signal_sell=0;
   for(int z=0;z<COUNT_PERIOD;z++)
     {
      double cur_open=iOpen(NULL,period2[z],0);
      colorbar=0;
      if(NewBar(period2[z])||prev==0)
        {
         bh[z] = cur_open;
         bl[z] = cur_open;
        }
      double cch=DiffCalculate(period2[z]);

      if(cur_open+cch>bh[z]) bh[z] = cur_open+cch;
      if(cur_open+cch<bl[z]) bl[z] = cur_open+cch;
      if(cch>0)
        {
         ExtHighBuffer[shift_bars+z]=bh[z]; ExtLowBuffer[shift_bars+z]=bl[z];
         signal_buy++;
         colorbar=1;
        }
      else
        {
         ExtHighBuffer[shift_bars+z]=bl[z]; ExtLowBuffer[shift_bars+z]=bh[z];
         signal_sell++;
         colorbar=2;
        }
        
      ExtOpenBuffer[shift_bars+z]=cur_open; ExtCloseBuffer[shift_bars+z]=cur_open+cch;
      ExtColorsBuffer[shift_bars+z]=colorbar;
      //if(ExtOpenBuffer[shift_bars+z]<ExtCloseBuffer[shift_bars+z])
      //   ExtColorsBuffer[shift_bars-z]=0;
      //if(ExtOpenBuffer[shift_bars+z]>ExtCloseBuffer[shift_bars+z])
      //   ExtColorsBuffer[shift_bars-z]=1;
      //if(ExtOpenBuffer[shift_bars+z]==ExtCloseBuffer[shift_bars+z])
      //   ExtColorsBuffer[shift_bars-z]=2;
//      comm=comm+string(ExtOpenBuffer[shift_bars+z])+"   "+string(ExtCloseBuffer[shift_bars+z])+"\n";
     }
   comm=comm+" Buy "+string(signal_buy)+" Sell "+string(signal_sell);
   Comment(comm);
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool NewBar(ENUM_TIMEFRAMES period)
  {
   MqlRates rt[1];
   if(CopyRates(_Symbol,period,0,1,rt)!=1)
     {
      Print("CopyRates of ",_Symbol," failed, no history");
      return(false);
     }
   if(rt[0].tick_volume>1) return(false);
   return(true);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int GetDelim(string Symb)
  {
   switch(StringLen(DoubleToString(SymbolInfoDouble(Symb,SYMBOL_BID),0)))
     {
      case 1: return(1);
      case 2: return(10);
      case 3: return(100);
      case 4: return(1000);
      case 5: return(10000);
      case 6: return(100000);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double DiffCalculate(ENUM_TIMEFRAMES period)
  {

   double ch=0;
   int cc=1;
//----
   for(int x=0;x<SymbolList.Total();x++)
     {
      if(SymbolInfoInteger(SymbolList.At(x),SYMBOL_TIME)<SymbolInfoInteger(_Symbol,SYMBOL_TIME) || Symbol()==SymbolList.At(x)) continue;
      for(int y=0;y<COUNT_PERIOD;y++)
        {
         double corr=MTF_MC_Correlation(_Symbol,SymbolList.At(x),period2[y]);
         if(corr==0.0) continue;
         double res=0;
         //if(corr>0.95)
         if(corr>InpCrl)
           {
            res=(SymbolInfoDouble(SymbolList.At(x),SYMBOL_BID)-iOpen(SymbolList.At(x),period,0))/GetDelim(SymbolList.At(x));
            if(res==0.0) continue;
            ch=ch+res;
            cc++;
            if(StringFind(corr_instr_arr,SymbolList.At(x)+string(period2[y]),0)>=0) continue;
            corr_instr_arr=corr_instr_arr+SymbolList.At(x)+string(period2[y]);

            comm=comm+SymbolList.At(x)+" TF "+PeriodToString(period2[y])+" Corr = "+DoubleToString(corr,5)+" Diff = "+string(res)+"\n";

           }
         //if(corr<-0.95)
         if(corr<-InpCrl)
           {
            res=(iOpen(SymbolList.At(x),period,0)-SymbolInfoDouble(SymbolList.At(x),SYMBOL_BID))/GetDelim(SymbolList.At(x));
            if(res==0.0) continue;
            ch=ch+res;
            //comment = comment + SymbolList.At(x) + " " + DoubleToStr(res,5) + "\n";
            cc++;
            if(StringFind(corr_instr_arr,SymbolList.At(x)+string(period2[y]),0)>=0) continue;
            corr_instr_arr=corr_instr_arr+SymbolList.At(x)+string(period2[y]);

            comm=comm+SymbolList.At(x)+" TF "+PeriodToString(period2[y])+" Corr = "+DoubleToString(corr,5)+" Diff = "+string(res)+"\n";
           }

        }
     }
//----
   return(ch/cc);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double MTF_MC_Correlation(string symbol1,string symbol2,ENUM_TIMEFRAMES period)
  {
   double sx,sy,cor1,cor2,cor3,z;
   MqlRates rates_symbol1[];
   if(CopyRates(symbol1,period,0,InpNumBarsHist,rates_symbol1)!=InpNumBarsHist)
     {
      return(0);
     }
   MqlRates rates_symbol2[];
   if(CopyRates(symbol2,period,0,InpNumBarsHist,rates_symbol2)!=InpNumBarsHist)
     {
      return(0);
     }

   sx=0; sy=0;
   for(int k=0; k<InpNumBarsHist; k++)
     {
      double x=0;
      double y=0;

      x=(rates_symbol1[k].open+rates_symbol1[k].close+rates_symbol1[k].high+rates_symbol1[k].low)/4;
      y=(rates_symbol2[k].open+rates_symbol2[k].close+rates_symbol2[k].high+rates_symbol2[k].low)/4;
      sx+=x;sy+=y;
     }
   sx/=InpNumBarsHist;sy/=InpNumBarsHist;
   cor1=0;cor2=0;cor3=0;
   for(int k=0; k<InpNumBarsHist; k++)
     {
      double x=0;
      double y=0;

      x=(rates_symbol1[k].open+rates_symbol1[k].close+rates_symbol1[k].high+rates_symbol1[k].low)/4;
      y=(rates_symbol2[k].open+rates_symbol2[k].close+rates_symbol2[k].high+rates_symbol2[k].low)/4;

      cor1+=(x-sx)*(y-sy);
      cor2+=(x-sx)*(x-sx);
      cor3+=(y-sy)*(y-sy);
     }
   ArrayFree(rates_symbol1);
   ArrayFree(rates_symbol2);
//----
   return(cor1/MathSqrt(cor2)/MathSqrt(cor3));
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
string PeriodToString(ENUM_TIMEFRAMES period)
  {
   switch(period)
     {
      case PERIOD_M1: return("M1");
      case PERIOD_M2: return("M2");
      case PERIOD_M3: return("M3");
      case PERIOD_M4: return("M4");
      case PERIOD_M5: return("M5");
      case PERIOD_M6: return("M6");
      case PERIOD_M10: return("M10");
      case PERIOD_M12: return("M12");
      case PERIOD_M15: return("M15");
      case PERIOD_M20: return("M20");
      case PERIOD_M30: return("M30");
      case PERIOD_H1: return("H1");
      case PERIOD_H2: return("H2");
      case PERIOD_H3: return("H3");
      case PERIOD_H4: return("H4");
      case PERIOD_H6: return("H6");
      case PERIOD_H8: return("H8");
      case PERIOD_H12: return("H12");
      case PERIOD_D1: return("D1");
      case PERIOD_W1: return("W1");
      case PERIOD_MN1: return("MN1");
     }
   return(NULL);
  };
//+------------------------------------------------------------------+
